// Copyright (c) 2022 NetEase, Inc. All rights reserved.
// Use of this source code is governed by a MIT license that can be
// found in the LICENSE file.

import 'dart:convert';

import 'package:hawk/hawk.dart';
import 'package:nim_core/nim_core.dart';

class HandleQChatMessageCase extends HandleBaseCase {
  HandleQChatMessageCase();

  @override
  Future<ResultBean?> handle(event) async {
    super.handle(event);
    var handled = false;
    var isCallback = false;
    var result;
    if (className != 'QChatMessageService') {
      return null;
    }
    var map = params![0];
    switch (methodName) {
      case 'sendMessage':
        var param = QChatSendMessageParam.fromJson(map);
        if (param.type == NIMMessageType.video && param.attach == "videoTest") {
          var fileAttachment = NIMVideoAttachment(
              size: 100,
              displayName: "test",
              url:
                  "https://nim-nosdn.netease.im/MjYxNDkzNzE=/bmltYV8xMTc5NjEyNzk0MjFfMTY2NjU3NjYyOTg0NV9jNGY5YzRjNS0yOWU1LTQ3YzMtOTE3Zi1kYTEwMWE5NmNkOGE=?createTime=1666580554897");

          param.attach = jsonEncode(fileAttachment.toMap());
        }
        final ret =
            await NimCore.instance.qChatMessageService.sendMessage(param);
        result = ret;
        handled = true;
        break;
      case 'resendMessage':
        final ret = await NimCore.instance.qChatMessageService
            .resendMessage(QChatResendMessageParam.fromJson(map));
        result = ret;
        handled = true;
        break;
      case 'downloadAttachment':
        final ret = await NimCore.instance.qChatMessageService
            .downloadAttachment(QChatDownloadAttachmentParam.fromJson(map));
        result = ret;
        handled = true;
        break;
      case 'getMessageHistory':
        final ret = await NimCore.instance.qChatMessageService
            .getMessageHistory(QChatGetMessageHistoryParam.fromJson(map));
        result = ret;
        handled = true;
        break;
      case 'updateMessage':
        final ret = await NimCore.instance.qChatMessageService
            .updateMessage(QChatUpdateMessageParam.fromJson(map));
        result = ret;
        handled = true;
        break;
      case 'revokeMessage':
        final ret = await NimCore.instance.qChatMessageService
            .revokeMessage(QChatRevokeMessageParam.fromJson(map));
        result = ret;
        handled = true;
        break;
      case 'deleteMessage':
        final ret = await NimCore.instance.qChatMessageService
            .deleteMessage(QChatDeleteMessageParam.fromJson(map));
        result = ret;
        handled = true;
        break;
      case 'markMessageRead':
        final ret = await NimCore.instance.qChatMessageService
            .markMessageRead(QChatMarkMessageReadParam.fromJson(map));
        result = ret;
        handled = true;
        break;
      case 'sendSystemNotification':
        final ret = await NimCore.instance.qChatMessageService
            .sendSystemNotification(
                QChatSendSystemNotificationParam.fromJson(map));
        result = ret;
        handled = true;
        break;
      case 'resendSystemNotification':
        final ret = await NimCore.instance.qChatMessageService
            .resendSystemNotification(
                QChatResendSystemNotificationParam.fromJson(map));
        result = ret;
        handled = true;
        break;
      case 'updateSystemNotification':
        final ret = await NimCore.instance.qChatMessageService
            .updateSystemNotification(
                QChatUpdateSystemNotificationParam.fromJson(map));
        result = ret;
        handled = true;
        break;
      case 'markSystemNotificationsRead':
        final ret = await NimCore.instance.qChatMessageService
            .markSystemNotificationsRead(
                QChatMarkSystemNotificationsReadParam.fromJson(map));
        result = ret;
        handled = true;
        break;
      case 'getMessageHistoryByIds':
        final ret = await NimCore.instance.qChatMessageService
            .getMessageHistoryByIds(
                QChatGetMessageHistoryByIdsParam.fromJson(map));
        result = ret;
        handled = true;
        break;
    }
    if (handled) {
      print('qChatMessageService#$methodName result: ${result.code}');
      handleCaseResult = ResultBean(
        code: result.code,
        data: result.data?.toJson(),
        message: result.errorDetails,
      );
    }
    if (isCallback) {
      handleCaseResult = ResultBean.success(data: null);
    }
    return handleCaseResult;
  }
}
